package com.sd.business.service.sku.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.sku.dto.SkuSelectDto;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.vo.SkuSpecVo;
import com.sd.business.entity.sku.vo.SkuVo;
import com.sd.business.mapper.sku.SkuMapper;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * sku 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-09
 */
@Service
public class SkuServiceImpl extends ServiceImpl<SkuMapper, Sku> implements SkuService {

    @Autowired
    private SkuSpecService skuSpecService;

    @Override
    public Page<SkuVo> getPage(SkuSelectDto dto) {

        // sku规格品号品名查询
        List<Long> selectSkuIdList = null;
        if (StrUtil.isNotBlank(dto.getSpecCode()) || StrUtil.isNotBlank(dto.getSpecName())) {
            selectSkuIdList = skuSpecService.listObject(SkuSpec::getSkuId, q -> q
                    .like(StrUtil.isNotBlank(dto.getSpecCode()), SkuSpec::getCode, dto.getSpecCode())
                    .like(StrUtil.isNotBlank(dto.getSpecName()), SkuSpec::getName, dto.getSpecName()));
            if (selectSkuIdList.isEmpty()) {
                return new Page<>();
            }
        }

        // sku分页
        Page<SkuVo> page = Sql.create(SkuVo.class)
                .selectAll(Sku.class)
                .from(Sku.class)
                .in(Sku::getId, selectSkuIdList)
                .like(Sku::getCode, dto.getCode())
                .like(Sku::getName, dto.getName())
                .eq(Sku::getType, dto.getType())
                .eq(Sku::getBrand, dto.getBrand())
                .orderByDesc(Sku::getId)
                .page(dto);

        List<SkuVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        // 赋值sku规格
        List<Long> skuIdList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<SkuSpec>> skuSpecMap = skuSpecService.mapKGroup(SkuSpec::getSkuId, q -> q
                .in(SkuSpec::getSkuId, skuIdList)
                .like(StrUtil.isNotBlank(dto.getSpecCode()), SkuSpec::getCode, dto.getSpecCode())
                .like(StrUtil.isNotBlank(dto.getSpecName()), SkuSpec::getName, dto.getSpecName()));
        records.forEach(item -> item.setSkuSpecList(skuSpecMap.getOrDefault(item.getId(), Collections.emptyList())));

        return page;
    }

    @Override
    public SkuVo detail(Long id) {

        // sku详情
        SkuVo vo = Sql.create(SkuVo.class)
                .selectAll(Sku.class)
                .from(Sku.class)
                .eq(Sku::getId, id)
                .one();
        Assert.notNull(vo, "未知数据");

        // 赋值sku规格
        List<SkuSpecVo> skuSpecList = Sql.create(SkuSpecVo.class)
                .selectAll(SkuSpec.class)
                .selectAs(ArtworkLibrary::getImgUrl, SkuSpecVo::getDesignImgUrl)
                .selectAs(ArtworkLibrary::getFileUrl, SkuSpecVo::getSharedFolder)
                .from(SkuSpecVo.class)
                .leftJoin(ArtworkLibrary.class, ArtworkLibrary::getId, SkuSpec::getArtworkLibraryId)
                .eq(SkuSpec::getSkuId, id)
                .list();
        vo.setSkuSpecList(skuSpecList);

        return vo;
    }

//    @Override
//    public void add(SkuDto dto) {
//        save(dto);
//    }
//
//    @Override
//    public void edit(SkuDto dto) {
//        updateById(dto);
//    }
//
//    @Override
//    public void delete(Long id) {
//        removeById(id);
//    }

}
