package com.sd.business.service.warehouse.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.warehouse.dto.WarehouseDto;
import com.sd.business.entity.warehouse.dto.WarehouseSelectDto;
import com.sd.business.entity.warehouse.po.Warehouse;
import com.sd.business.entity.warehouse.vo.WarehouseVo;
import com.sd.business.mapper.warehouse.WarehouseMapper;
import com.sd.business.service.warehouse.WarehouseService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.stereotype.Service;


/**
 * <p>
 * 仓库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-07
 */
@Service
public class WarehouseServiceImpl extends ServiceImpl<WarehouseMapper, Warehouse> implements WarehouseService {

    @Override
    public Page<WarehouseVo> getPage(WarehouseSelectDto dto) {

        Page<WarehouseVo> page = Sql.create(WarehouseVo.class)
                .selectAll(Warehouse.class)
                .from(Warehouse.class)
                .eq(Warehouse::getType, dto.getType())
                .like(Warehouse::getName, dto.getName())
                .orderByDesc(Warehouse::getId)
                .page(dto);

        return page;
    }

    @Override
    public WarehouseVo detail(Long id) {

        WarehouseVo vo = Sql.create(WarehouseVo.class)
                .selectAll(Warehouse.class)
                .from(Warehouse.class)
                .eq(Warehouse::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(WarehouseDto dto) {
        save(dto);
    }

    @Override
    public void edit(WarehouseDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

}
