package com.sd.business.upload;


import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.obs.services.ObsClient;
import com.ruoyi.common.exception.ServiceException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.ByteArrayInputStream;
import java.util.Date;
import java.util.StringJoiner;

/**
 * 文件上传服务
 */
@Component
public class ObsUploadService {

    @Value("${obs.ak}")
    private String ak;

    @Value("${obs.sk}")
    private String sk;

    @Value("${obs.endPoint}")
    private String endPoint;

    @Value("${obs.url}")
    private String url;

    @Value("${obs.bucketName}")
    private String bucketName;

    @Value("${spring.profiles.active}")
    private String active;

    @Value("${ruoyi.name}")
    private String name;

    public String uploadFileByte(ByteArrayInputStream bis, String fileName) {

        if (fileName.length() > 100) {
            throw new ServiceException("文件名过长,请缩短后重试！！！");
        }

        // 文件后缀名
        String suffix = FileUtil.getSuffix(fileName);

        // 文件路径
        String objectKey = new StringJoiner("/")
                .add(name)
                .add(active)
                .add(DateUtil.format(new Date(), "yyyy/MM/dd"))
                .add(IdUtil.fastSimpleUUID() + (ObjectUtil.isEmpty(suffix) ? "" : "." + suffix))
                .toString();

        ObsClient obsClient = null;

        try {
            // 获取oss链接客户端
            obsClient = getObsClient();
            // 上传
            obsClient.putObject(bucketName, objectKey, bis);
        } catch (Exception e) {
            throw new ServiceException("上传失败");
        } finally {
            IoUtil.close(obsClient);
            IoUtil.close(bis);
        }

        return objectKey;
    }

    /**
     * 获取oss链接客户端
     */
    private ObsClient getObsClient() {
        return new ObsClient(ak, sk, endPoint);
    }

}
