package com.sd.framework.aspect;

import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Aspect
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class DataSourceAspect {

    @Around(value = "execution (* com.*.*.service.*.*.*(..)) || execution (* com.*.*.service.*.*(..)) || execution(* com.*.*.flow.*.*(..))")
    public Object before(ProceedingJoinPoint point) throws Throwable {
        Object target = point.getTarget();
        String name = target.getClass().getName();
        String moduleName = name.split("\\.")[1];

        String peek = DynamicDataSourceContextHolder.peek();
        if ("sd".equals(moduleName)) {
            if ("business".equals(peek)) {
                return point.proceed();
            } else {
                try {
                    DynamicDataSourceContextHolder.push("business");
                    return point.proceed();
                } finally {
                    DynamicDataSourceContextHolder.poll();
                }
            }
        } else {
            if ("base".equals(peek)) {
                return point.proceed();
            } else {
                try {
                    DynamicDataSourceContextHolder.push("base");
                    return point.proceed();
                } finally {
                    DynamicDataSourceContextHolder.poll();
                }
            }
        }
    }

}
