package com.sd.framework.util.packing;


import com.sd.framework.util.packing.entity.Instance;
import com.sd.framework.util.packing.entity.PlaceSquare;
import com.sd.framework.util.packing.entity.Solution;
import com.sd.framework.util.packing.model.TabuSearch;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

import java.util.Random;

/**
 * 二维装箱禁忌搜索算法
 */
public class Application extends javafx.application.Application {

    private int counter = 0;

    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage primaryStage) throws Exception {

        String path = "src/main/resources/data.txt";

        TabuSearch model = new TabuSearch(ReadDataUtil.getInstance(path));
        Solution solution = model.search();

        Instance instance = solution.getInstance();
        AnchorPane pane = new AnchorPane();
        plot(solution);
        Canvas canvas = new Canvas(instance.getLength(), instance.getWidth());
        pane.getChildren().add(canvas);
        canvas.relocate(100, 100);
        // 绘制最外层的矩形
        draw(canvas, 0, 0, instance.getLength(), instance.getWidth());
        // 添加按钮
        Button nextButton = getButton(solution, canvas);
        pane.getChildren().add(nextButton);
        primaryStage.setTitle("二维下料可视化");
        primaryStage.setScene(new Scene(pane, 1000, 1000, Color.AQUA));
        primaryStage.show();
    }

    private Button getButton(Solution solution, Canvas canvas) {
        Button nextButton = new Button("Next");
        nextButton.setOnAction(actionEvent -> {
            try {
                PlaceSquare placeSquare = solution.getPlaceSquareList().get(counter);
                draw(canvas, placeSquare.getX(), placeSquare.getY(), placeSquare.getLength(), placeSquare.getWidth());
                counter++;
            } catch (Exception e) {
                Alert alert = new Alert(Alert.AlertType.WARNING);
                alert.setContentText("已经没有可以放置的矩形了！");
                alert.showAndWait();
            }
        });
        return nextButton;
    }

    private void plot(Solution solution) {
        System.out.println(solution);
    }

    private void draw(Canvas canvas, double x, double y, double length, double width) {
        GraphicsContext gc = canvas.getGraphicsContext2D();
        // 边框
        gc.setStroke(Color.BLACK);
        gc.setLineWidth(2);
        gc.strokeRect(x, y, length, width);
        // 填充
        gc.setFill(new Color(new Random().nextDouble(), new Random().nextDouble(), new Random().nextDouble(), new Random().nextDouble()));
        gc.fillRect(x, y, length, width);
    }

}

