package com.sd.framework.util.tree;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

public class TreeUtil {

    /**
     * 构件树形集合
     *
     * @param list 所有节点
     * @return 树形集合
     */
    public static <T extends Tree<T>> List<T> build(List<T> list) {
        // 获取节点id列表
        List<Long> parentIdList = list.stream().map(Tree::getId).collect(Collectors.toList());
        return list.stream()
                // 过滤非顶级节点
                .filter(item -> !parentIdList.contains(item.getParentId()))
                // 赋值顶级节点子节点
                .peek(item -> item.setChildren(findChildNodes(item.getId(), list)))
                // 收集数据
                .collect(Collectors.toList());
    }

    /**
     * 获取子节点id列表
     *
     * @param id   父节点id
     * @param list 所有节点id列表
     * @return 子节点id列表
     */
    public static <T extends TreeId> List<Long> getChildrenIdList(Long id, List<T> list) {
        List<Long> result = list.stream()
                .filter(item -> Objects.equals(id, item.getParentId()))
                .flatMap(item -> getChildrenIdList(item.getId(), list).stream())
                .distinct()
                .collect(Collectors.toList());

        result.add(0, id);

        return result;
    }

    /**
     * 查找子节点集合
     *
     * @param id   节点id
     * @param list 所有节点
     * @return 子节点结果集
     */
    private static <T extends Tree<T>> List<T> findChildNodes(Long id, List<T> list) {
        return list.stream()
                // 过滤非子节点
                .filter(item -> Objects.equals(id, item.getParentId()))
                // 递归赋值子节点的子节点
                .peek(item -> item.setChildren(findChildNodes(item.getId(), list)))
                // 收集数据
                .collect(Collectors.toList());
    }

}
