package com.sd.wln.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.framework.mybatis.holder.LogicHolder;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuClassify;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.service.sku.SkuClassifyService;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.wln.service.WlnSkuService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WlnSkuServiceImpl implements WlnSkuService {

    public static final Lock syncSkuLock = new ReentrantLock();

    @Autowired
    private SkuClassifyService skuClassifyService;

    @Autowired
    private SkuService skuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void sync() {
        if (syncSkuLock.tryLock()) {
            try {
                syncSkuClassify();
                syncSku();
            } finally {
                syncSkuLock.unlock();
            }
        }
    }

    private void syncSkuClassify() {
        List<JSONObject> list;

        // 查询万里牛sku分类
        try {
            list = WlnUtil.getSkuClassifyList();
        } catch (Exception e) {
            log.error("sku分类同步失败", e);
            throw new ServiceException("sku分类同步失败");
        }

        // 查询数据库sku分类
        LogicHolder.setLogicHolder(true);
        Map<String, SkuClassify> skuClassifyMap = skuClassifyService.mapKEntity(SkuClassify::getWlnCatagoryId, null);
        LogicHolder.clear();

        // 去重合并
        List<SkuClassify> skuClassifyList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            JSONObject item = list.get(i);

            String catagoryid = item.getString("catagoryid");
            String name = item.getString("catagory_name");
            String parentId = item.getString("parentid");

            SkuClassify skuClassify = skuClassifyMap.get(catagoryid);
            if (skuClassify == null) {
                skuClassify = new SkuClassify();
                skuClassify.setId(IdWorker.getId());
            }
            skuClassify.setWlnCatagoryId(catagoryid);
            skuClassify.setName(name);
            skuClassify.setWlnParentId(parentId);
            skuClassify.setSort(i);
            skuClassifyList.add(skuClassify);
        }

        // 赋值parentId
        Map<String, Long> map = skuClassifyList.stream().collect(
                Collectors.toMap(SkuClassify::getWlnCatagoryId, SkuClassify::getId, (t1, t2) -> t1));
        for (SkuClassify skuClassify : skuClassifyList) {
            skuClassify.setParentId(map.getOrDefault(skuClassify.getWlnParentId(), 0L));
        }

        // 更新数据库
        skuClassifyService.saveOrUpdateBatch(skuClassifyList);
    }

    private void syncSku() {
        Sku tempSku = skuService.getOne(q -> q.orderByDesc(BasePo::getUpdateTime));
        Long modifyTime;
        if (tempSku == null) {
            modifyTime = 965381503L;
        } else {
            modifyTime = tempSku.getUpdateTime().getTime() + 1;
        }

        int page = 1;
        int size;
        List<JSONObject> wlnSkuList = new ArrayList<>();
        do {
            try {
                List<JSONObject> itemList = WlnUtil.getSkuList(page, 200, modifyTime);
                wlnSkuList.addAll(itemList);
                page++;
                size = itemList.size();
            } catch (Exception e) {
                log.error("sku同步失败", e);
                throw new ServiceException("sku同步失败");
            }
        } while (size == 200);

        if (wlnSkuList.isEmpty()) {
            return;
        }

        LogicHolder.setLogicHolder(true);

        // 产品分类map
        Map<String, Long> skuClassifyMap = skuClassifyService.mapKV(SkuClassify::getWlnCatagoryId, BaseIdPo::getId, null);

        // 产品map
        Map<String, Sku> skuMap = skuService.mapKEntity(Sku::getWlnUid, null);

        // 产品规格map
        Map<Long, Map<String, SkuSpec>> specMap = skuSpecService.list(q -> q.isNotNull(SkuSpec::getWlnUid)).stream()
                .collect(Collectors.groupingBy(SkuSpec::getSkuId, Collectors.toMap(SkuSpec::getWlnUid, Function.identity())));

        LogicHolder.clear();

        List<Sku> newSkuList = new ArrayList<>();
        List<SkuSpec> newSkuSpecList = new ArrayList<>();

        for (JSONObject wlnSku : wlnSkuList) {
            Sku sku = skuMap.get(wlnSku.getString("sys_goods_uid"));
            if (sku == null) {
                sku = createSku(wlnSku, skuClassifyMap);
                if (sku == null) {
                    continue;
                }
                newSkuList.add(sku);
            }

            List<JSONObject> specList = wlnSku.getJSONArray("specs").toJavaList(JSONObject.class);
            for (JSONObject wlnSkuSpec : specList) {
                Map<String, SkuSpec> skuSpecMap = specMap.get(sku.getId());
                if (skuSpecMap != null && skuSpecMap.get(wlnSkuSpec.getString("sys_spec_uid")) != null) {
                    continue;
                }
                newSkuSpecList.add(createSkuSpec(wlnSkuSpec, sku));
            }
        }

        skuService.saveBatch(newSkuList);
        skuSpecService.saveBatch(newSkuSpecList);
    }

    private Sku createSku(JSONObject wlnSku, Map<String, Long> skuClassifyMap) {

        String catagoryId = wlnSku.getString("catagory_id");
        if (StrUtil.isBlank(catagoryId)) {
            return null;
        }

        Long skuClassifyId = skuClassifyMap.get(catagoryId);
        if (skuClassifyId == null) {
            return null;
        }

        Sku sku = new Sku();
        sku.setId(IdWorker.getId());
        sku.setWlnCatagoryId(catagoryId);
        sku.setWlnUid(wlnSku.getString("sys_goods_uid"));
        sku.setSource(2);
        sku.setSkuClassifyId(skuClassifyId);
        sku.setCode(wlnSku.getString("goods_code"));
        sku.setName(wlnSku.getString("goods_name"));
        sku.setBrand(wlnSku.getString("brand_name"));
        sku.setMainImgUrl(wlnSku.getString("pic"));
        sku.setType(Objects.equals(sku.getBrand(), "胜德科技") ? 1 : 0);
        return sku;
    }

    private SkuSpec createSkuSpec(JSONObject wlnSku, Sku sku) {
        SkuSpec skuSpec = new SkuSpec();
        skuSpec.setId(IdWorker.getId());
        skuSpec.setWlnUid(wlnSku.getString("sys_spec_uid"));
        skuSpec.setSkuId(sku.getId());
        skuSpec.setSpecImgUrl(wlnSku.getString("pic"));
        skuSpec.setCode(wlnSku.getString("spec_code"));
        skuSpec.setName(sku.getName() + ":" + wlnSku.getString("spec1") + wlnSku.getString("spec2"));
        skuSpec.setBarCode(wlnSku.getString("barcode"));
        skuSpec.setLength(wlnSku.getBigDecimal("length"));
        skuSpec.setWidth(wlnSku.getBigDecimal("width"));
        skuSpec.setHeight(wlnSku.getBigDecimal("height"));
        skuSpec.setNetWeight(wlnSku.getBigDecimal("weight"));
        skuSpec.setRemark(wlnSku.getString("spec1") + wlnSku.getString("spec2"));
        return skuSpec;
    }

}
